<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Extended Product Grid Subscription Functionality
 */

namespace Amasty\PgridSubscriptionFunctionality\Model\ResourceModel\Product;

use Amasty\PgridSubscriptionFunctionality\Ui\DataProvider\Product\AddQtyPerSourceFilterToCollection;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Framework\App\ResourceConnection;

class AddSalableQtyToCollection
{
    /**
     * @var ResourceConnection
     */
    private $resource;

    public function __construct(
        ResourceConnection $resource
    ) {
        $this->resource = $resource;
    }

    public function execute(Collection $collection): void
    {
        $select = $collection->getSelect();
        $subSelect = $this->resource->getConnection()->select()
            ->from(
                $this->resource->getTableName('inventory_reservation'),
                ['reserved_quantity' => new \Zend_Db_Expr('SUM(quantity)'), 'sku', 'stock_id']
            )->group(['sku', 'stock_id']);

        if (!$collection->getFlag(AddQtyPerSourceFilterToCollection::JOIN_FLAG)) {
            $select->joinLeft(
                ['isi' => $this->resource->getTableName('inventory_source_item')],
                'e.sku = isi.sku',
                'quantity'
            );
            $collection->setFlag(AddQtyPerSourceFilterToCollection::JOIN_FLAG, true);
        }

        $select->joinLeft(
            ['ss' => $this->resource->getTableName('inventory_source_stock_link')],
            'isi.source_code = ss.source_code',
            'stock_id'
        )->joinLeft(
            ['csi' => $this->resource->getTableName('cataloginventory_stock_item')],
            'csi.product_id = e.entity_id AND csi.stock_id = ss.stock_id',
            'min_qty'
        )->joinLeft(
            ['res' => $subSelect],
            'res.sku = e.sku AND ss.stock_id = res.stock_id',
            'reserved_quantity'
        )->columns([
            'salable_qty' => $this->getSalableQtyColumnExpr()
        ])->group('e.sku');
    }

    public function getSalableQtyColumnExpr(): \Zend_Db_Expr
    {
        return new \Zend_Db_Expr(
            '(isi.quantity + IFNULL(res.reserved_quantity, 0) - IFNULL(csi.min_qty, 0))'
        );
    }
}
