<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Extended Product Grid Subscription Functionality
 */

namespace Amasty\PgridSubscriptionFunctionality\Plugin\Pgrid\Plugin\Catalog\Ui\DataProvider\Product\ProductDataProvider;

use Amasty\Pgrid\Plugin\Catalog\Ui\DataProvider\Product\ProductDataProvider;
use Amasty\PgridSubscriptionFunctionality\Model\ConfigProvider;

class CropDecimalDigits
{
    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @var array
     */
    private $columnKeysToSkip;

    public function __construct(
        ConfigProvider $configProvider,
        array $columnKeysToSkip = []
    ) {
        $this->configProvider = $configProvider;
        $this->columnKeysToSkip = $columnKeysToSkip;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterAfterGetData(ProductDataProvider $subject, $result)
    {
        if ($this->configProvider->isCropDecimal()) {
            $this->processFloatValues($result);
        }

        return $result;
    }

    private function processFloatValues(array &$result): void
    {
        if (!isset($result['items'])) {
            return;
        }

        foreach ($result['items'] as &$item) {
            foreach ($item as $key => $value) {
                if (array_key_exists($key, $this->columnKeysToSkip)) {
                    continue;
                }
                if (is_numeric($value) && is_float($value + 0)) {
                    $item[$key] = $this->round((float)$value);
                }
            }
        }
    }

    private function round(float $value): string
    {
        $roundedValue = round($value, 2);
        $decimalPart = (string)($roundedValue * 100 % 100);
        if ($decimalPart === '0') {
            $decimalPart = '00';
        }

        return (int)$roundedValue . '.' . $decimalPart;
    }
}
