<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Extended Product Grid Subscription Functionality
 */

namespace Amasty\PgridSubscriptionFunctionality\Ui\DataProvider\Product;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Data\Collection;
use Magento\Ui\DataProvider\AddFilterToCollectionInterface;

class AddQtyPerSourceFilterToCollection implements AddFilterToCollectionInterface
{
    public const JOIN_FLAG = 'isi_joined';

    /**
     * @var ResourceConnection
     */
    private $resource;

    public function __construct(
        ResourceConnection $resource
    ) {
        $this->resource = $resource;
    }

    public function addFilter(Collection $collection, $field, $condition = null)
    {
        if (!$collection->getFlag(self::JOIN_FLAG)) {
            $collection->getSelect()->join(
                ['isi' => $this->resource->getTableName('inventory_source_item')],
                'e.sku = isi.sku',
                'isi.quantity'
            )->group('e.sku');
            $collection->setFlag(self::JOIN_FLAG, true);
        }

        if (isset($condition['gteq'])) {
            $collection->getSelect()->where(
                'isi.quantity >= ?',
                (float)$condition['gteq']
            );
        }
        if (isset($condition['lteq'])) {
            $collection->getSelect()->where(
                'isi.quantity <= ?',
                (float)$condition['lteq']
            );
        }
    }
}
