<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Extended Product Grid Subscription Functionality
 */

namespace Amasty\PgridSubscriptionFunctionality\Ui\DataProvider\Product;

use Amasty\PgridSubscriptionFunctionality\Model\ResourceModel\Product\AddSalableQtyToCollection;
use Magento\Framework\Data\Collection;
use Magento\Ui\DataProvider\AddFilterToCollectionInterface;

class AddSalableQtyFilterToCollection implements AddFilterToCollectionInterface
{
    public const JOIN_FLAG = 'salable_qty_joined';

    /**
     * @var AddSalableQtyToCollection
     */
    private $addSalableQtyToCollection;

    public function __construct(
        AddSalableQtyToCollection $addSalableQtyToCollection
    ) {
        $this->addSalableQtyToCollection = $addSalableQtyToCollection;
    }

    public function addFilter(Collection $collection, $field, $condition = null)
    {
        if (!$collection->getFlag(self::JOIN_FLAG)) {
            $this->addSalableQtyToCollection->execute($collection);
            $collection->setFlag(self::JOIN_FLAG, true);
        }

        if (isset($condition['gteq'])) {
            $collection->getSelect()->where(
                $this->addSalableQtyToCollection->getSalableQtyColumnExpr() . ' >= ?',
                (float)$condition['gteq']
            );
        }
        if (isset($condition['lteq'])) {
            $collection->getSelect()->where(
                $this->addSalableQtyToCollection->getSalableQtyColumnExpr() . ' <= ?',
                (float)$condition['lteq']
            );
        }
    }
}
